import React, { useState } from "react";
import styled from "styled-components";
import ClickToCopy from "../../util/ClickToCopy";
import Button from "../../util/Button";
import Input from "../../util/Input";
import Loading from "../../util/Loading";
import LobbyPlayerList from "./LobbyPlayerList";
import DelayedButton from "../../util/DelayedButton";
import HeaderAndFooter from "../../util/HeaderAndFooter";
import { dispatch, useGameId, usePlayerName, useGameJoined } from "../../../domain/gameStore";

const InfoContainer = styled.div`
  flex: 3;
  align-self: center;

  display: flex;
  flex-flow: column nowrap;
  justify-content: center;
  align-items: center;
`

const Label = styled.span`
  padding: 0.2em;
`

const FormInput = styled(Input)`
  padding: 0.5em;
`

const FormButton = styled(Button)`
  padding: 1em;
  margin: 0.2em;
  margin-top: 0.3em;
`

const StyledDelayButton = styled(DelayedButton)`
  padding: 1em;
  margin: 0.2em;
  margin-top: 0.3em;
`

const PageContainer = styled.div`
  flex: 1;
  display: flex;
  flex-flow: row nowrap;
  justify-content: space-between;
  align-items: flex-start;
`

const getUrl = gameId => {
  const u = new URL(window.location.href);
  u.searchParams.append("join", gameId);
  return u.href;
}

const LobbyInfo = () => {
  const gameId = useGameId();
  const playerName = usePlayerName();

  return (
    <>
      <Label>Playing as {playerName}</Label>
      <Label>Use this link to invite other players:</Label>
      <Label><ClickToCopy text={getUrl(gameId)} /></Label>
      <StyledDelayButton 
        onEnd={dispatch.startRound} 
        countDownFormatter={rem => `Click to cancel, ${rem}s...`}
      >
        Start Game
      </StyledDelayButton>
    </>
  )
};

const JoinForm = () => {
  const gameId = useGameId();
  const playerName = usePlayerName();
  const [loading, setLoading] = useState(false);
  const [failed, setFailed] = useState(false);

  if (loading) {
    return <Loading/>
  }

  const onJoinGame = async () => {
    // TODO would like to support re-joining a game you left
    setLoading(true);
    try {
      await dispatch.joinGame();
    } catch (err) {
      // failed to join the game
      setFailed(true);
      setLoading(false);
      return;
    }
    setFailed(false);
  };
  const cannotJoinGame = !playerName || playerName.length === 0;

  return (
    <>
      {failed && <Label>Failed to join the game! Maybe try a different name?</Label>}
      <Label>Joining {gameId}...</Label>
      <Label>Player Name</Label>
      <FormInput
        type="text" 
        value={playerName || ""} 
        onChange={({ target }) => dispatch.setPlayerName(target.value)} 
        onKeyDown={({ key }) => { if (key === "Enter") { onJoinGame() }}}
      />
      <FormButton onClick={onJoinGame} disabled={cannotJoinGame}>Join Game</FormButton>
    </>
  );
}

const Lobby = ({ onStart }) => {
  const joined = useGameJoined();

  return (
    <HeaderAndFooter>
      <PageContainer>
        <InfoContainer>
          {
            joined
              ? <LobbyInfo onStart={onStart}/>
              : <JoinForm />
          }
        </InfoContainer>
        <LobbyPlayerList/>
      </PageContainer>
    </HeaderAndFooter>
  )
};

export default Lobby;
